@@ -190,10 +190,7 @@ static const registry<gdbarch>::key<windows_gdbarch_data>
 static struct windows_gdbarch_data *
 get_windows_gdbarch_data (struct gdbarch *gdbarch)
 {
-  windows_gdbarch_data *result = windows_gdbarch_data_handle.get (gdbarch);
-  if (result == nullptr)
-    result = windows_gdbarch_data_handle.emplace (gdbarch);
-  return result;
+  return &windows_gdbarch_data_handle.try_emplace (gdbarch);
 }
 
 /* Define Thread Local Base pointer type.  */
@@ -930,6 +927,14 @@ windows_solib_ops::iterate_over_objfiles_in_search_order
       }
 }
 
+/* Implement the "auto_wide_charset" gdbarch method.  */
+
+static const char *
+windows_auto_wide_charset ()
+{
+  return "UTF-16";
+}
+
 /* Common parts for gdbarch initialization for the Windows and Cygwin OS
    ABIs.  */
 
@@ -938,6 +943,7 @@ windows_init_abi_common (struct gdbarch_info info, struct gdbarch *gdbarch)
 {
   set_gdbarch_wchar_bit (gdbarch, 16);
   set_gdbarch_wchar_signed (gdbarch, 0);
+  set_gdbarch_auto_wide_charset (gdbarch, windows_auto_wide_charset);
 
   /* Canonical paths on this target look like
      `c:\Program Files\Foo App\mydll.dll', for example.  */
@@ -954,6 +960,9 @@ windows_init_abi (struct gdbarch_info info, struct gdbarch *gdbarch)
 {
   windows_init_abi_common (info, gdbarch);
   set_gdbarch_gdb_signal_to_target (gdbarch, windows_gdb_signal_to_target);
+
+  /* On Windows, "long"s are only 32bit.  */
+  set_gdbarch_long_bit (gdbarch, 32);
 }
 
 /* See windows-tdep.h.  */
@@ -1103,11 +1112,9 @@ struct cpms_data
 };
 
 static void
-core_process_module_section (bfd *abfd, asection *sect, void *obj)
+core_process_module_section (bfd *abfd, asection *sect, cpms_data &data)
 {
-  struct cpms_data *data = (struct cpms_data *) obj;
-  enum bfd_endian byte_order = gdbarch_byte_order (data->gdbarch);
-
+  bfd_endian byte_order = gdbarch_byte_order (data.gdbarch);
   unsigned int data_type;
   char *module_name;
   size_t module_name_size;
@@ -1154,10 +1161,10 @@ core_process_module_section (bfd *abfd, asection *sect, void *obj)
   module_name = (char *) buf.data () + module_name_offset;
 
   /* The first module is the .exe itself.  */
-  if (data->module_count != 0)
+  if (data.module_count != 0)
     windows_xfer_shared_library (module_name, base_addr,
-				 NULL, data->gdbarch, data->xml);
-  data->module_count++;
+				 NULL, data.gdbarch, data.xml);
+  data.module_count++;
 }
 
 ULONGEST
@@ -1166,7 +1173,10 @@ windows_core_xfer_shared_libraries (struct gdbarch *gdbarch,
 				    ULONGEST offset, ULONGEST len)
 {
   cpms_data data { gdbarch, "<library-list>\n", 0 };
-  bfd_map_over_sections (&cbfd, core_process_module_section, &data);
+
+  for (asection *sect : gdb_bfd_sections (&cbfd))
+    core_process_module_section (&cbfd, sect, data);
+
   data.xml += "</library-list>\n";
 
   ULONGEST len_avail = data.xml.length ();
